/* 
 * 
 *	Michael L Anderson
 *	5/21/99 
 *	Section BA
 * CSE142, Spring 1999
 * Homework 4
 * Traffic animation

  STARTER PROGRAM

 *
 *
 */


#include "gp142.h"
#include <stdio.h>
#include <math.h>

#include <time.h>
	/* 
	* <time.h> is included so the current time can be accessed
	* to initialize the random number generator.  If you don't
	* use random numbers (and you don't have to) you can either
	* ignore this #include or delete it.
	*/

/* Logical constants */
#define TRUE    		1
#define FALSE   		0

/* 
 * For many of the GP142 functions, a line width of
 * zero means to fill the shape being drawn.  The
 * constant FILL is used in function calls when a solid
 * shape is desired.
 */
#define FILL 0


/*
 * Symbolic constants.  Change as needed.
 */

/* Menu buttons: */

/* Top-left coordinates of button palette */
#define MENU_TOP		(GP142_YMAX - 5)
#define MENU_LEFT		(-GP142_XMAX + 5)

/* Height and width of buttons in pixels */
#define BUTTON_HEIGHT  	25
#define BUTTON_WIDTH 	90

/* Number of buttons */
#define NUM_BUTTONS 	7


/* Possible locations of mouse clicks: */

/*   Mouse click in some button */
#define BUTTON_CLICK 1

/*   Mouse click elsewhere */
#define OTHER_CLICK 0


/* Coordinates of objects in the drawing */

/* y-coordinate of the top of the ground in the landscape */
#define HORIZON (- 2 * (GP142_YMAX / 10))

/* Vertical center and width of highway */
#define HIGHWAY_CENTER ((-GP142_YMAX + HORIZON) / 3)
#define HIGHWAY_WIDTH  10


/* Maximum number of vehicles that can appear in the animation */
#define MAX_VEHICLES	5

/* Height and width of a single car */
#define CAR_HEIGHT 24
#define CAR_WIDTH  40

/* Height and width of a tank */
#define TANK_HEIGHT 32
#define TANK_WIDTH 64

/* Number of animation frames for a moving projectile */
#define TOTAL_FRAMES 6

/*****************************************************************************

     FUNCTION PROTOTYPES

*****************************************************************************/

/* Initialize random number generator.  [Ignore or		*/
/* remove if your program doesn't use random numbers.]	*/
void initialize_random_numbers();

/* Draw the background, including the sun at coordinates sun_x and sun_y */
void draw_landscape(int sun_x, int sun_y);

/* Draw the buttons */
void draw_buttons();

/* Return kind of mouse click (in a button, etc.) */
int  classify_mouse_click(int mouse_x, int mouse_y);

/* Process button click at location x, y.  Update car speed,  */
/* direction, color, and whether traffic is moving as needed. */
void handle_button(int x, int y, int num_vehicles, int cardata[][8], int *tank_engaged);

/* Update sun coordinates sun_x, sun_y for one tick */
void move_sun(int *sun_x, int *sun_y);

/* Update all vehicle x, y coordinates, given current x, y */
/* location, direction (+1 or -1), and speed in pixels per tick. */
void move_traffic(int num_vehicles, int cardata[][8]);

/* Update tank coordinates x, y */
void move_tank(int *x, int *y, int target);

/* Draw all vehicles given their current x, y coordinates and color */
void draw_traffic(int num_vehicles, const cardata[][8]);

/* Draw a single car of the given color with horizontal */
/* center x and bottom y*/
void draw_car(int x, int y, int color, int selected, int alive);

/* Draw a single tank of the given color with horizontal */
/* center x and bottom y*/
void draw_tank(int x, int y, int target_x, int target_y);

/* Draw tank firing animation sequence */
void fire_tank(int x, int y, int target_x, int target_y, int frame);

/* Display error message if internal error is detected. */
void something_is_wrong (void);


/*****************************************************************************

     MAIN

*****************************************************************************/

/*
 * Main program contents:
 *   - Major program variables that persist throughout execution
 *   - Event loop that processes GP142 events until the user quits,
 *     and updates variables and redraws the window as needed.
 */
int main(void)
{
	/* Event handling and buttons: */
	int i;					/* multipurpose incrementation variable		*/
	int  quit;				/* = "user has selected quit"				*/
	int  event;				/* the most recent GP142 event				*/
	char key_pressed;		/* last keyboard character from key event	*/
	int  mouse_x, mouse_y;	/* x and y coordinates of latest mouse event*/
	int  click_kind;		/* kind of mouse click (button, etc.)		*/
	
	/* Information about the tank */	
	int  tank_x, tank_y;	/* x and y coordinates for the tank			*/
	int  target;			/* number of current target					*/
	int  tank_engaged;		/* whether the tank routine is engaged		*/
	int  tank_fired;		/* has the tank fired recently?				*/
	int  fire_frame;		/* how long ago has the tank fired?			*/
	int	 target_canidate_exists; /* Target canidate exists				*/

	/* Information about the sun: */
	int sun_x, sun_y;			/* screen x and y coordinates */

	/* Information about vehicles in the animation:		*/
	/* 0 = current x position of the car				*/
	/* 1 = current y position of the car				*/
	/* 2 = current direction: +1=right, -1=left			*/
	/* 3 = color of the car								*/
	/* 4 = speed in pixels per tick when traffic moving	*/
	/* 5 = car traffic is currently moving				*/
	/* 6 = the car is selected							*/
	/* 7 = the car is "alive"							*/
	int cardata[MAX_VEHICLES][8];
	int num_vehicles;
	
	
	initialize_random_numbers();
	
	/* Initialize graphics package, clear the window,	*/
	/* turn logging off, and start animation.			*/
	GP142_open();
	GP142_logging(LOG_OFF);
	GP142_clear();
	GP142_animate(ANI_RUN);

	/* set initial location of the sun */
	sun_x = GP142_XMAX - 50;
	sun_y = GP142_YMAX - 35;

	/* set initial conditions for the tank */
	tank_engaged = FALSE;
	tank_x = -GP142_XMAX - TANK_WIDTH/2;
	tank_y = -(3*GP142_YMAX/4);
	target = 0;
	tank_fired = FALSE;
	fire_frame = 0;

	/* set initial conditions for the car */
	num_vehicles = 5;
	for (i=0; i<num_vehicles; i++) {
		cardata[i][0] = (rand() % (GP142_XMAX*2))-GP142_XMAX;
		cardata[i][1] = HIGHWAY_CENTER;
		cardata[i][2] = 2*(rand() % 2)-1;
		cardata[i][3] = rand() % MAX_COLORS;
		cardata[i][4] = 5;
		cardata[i][5] = TRUE;
		cardata[i][6] = FALSE;
		cardata[i][7] = TRUE;
	}
		cardata[0][6] = TRUE;

	/* Draw initial landscape, buttons, and traffic */
	draw_landscape(sun_x,sun_y);
	draw_buttons( );
	draw_traffic(num_vehicles, cardata);
	

	/*
	 * Main event loop:
	 * ---- ----- -----
	 * Wait for the next user action, decode it, and call an
	 * appropriate function to handle it.  Repeat until the
	 * user selects quit.
	 */
	quit = FALSE;
	while (!quit) {
		
		/* get next event */
		event = GP142_await_event(&mouse_x, &mouse_y, &key_pressed);
		
		/* perform appropriate action depending on kind of event */
		switch (event) {
		case GP142_QUIT:
			/* Quit selected */
			quit = TRUE;
			break;
			
		case GP142_KBD:
			break;
			
		case GP142_MOUSE:
			/* Mouse click.  Determine if in a menu button	*/
			/* or elsewhere and process accordingly.		*/
			click_kind = classify_mouse_click(mouse_x, mouse_y);
			if (click_kind == BUTTON_CLICK) {
				handle_button(mouse_x, mouse_y, num_vehicles, cardata, &tank_engaged);
			} else 
				for (i=0; i<num_vehicles; i++) 
					if (mouse_x<(cardata[i][0]+CAR_WIDTH+2) && mouse_x>(cardata[i][0]-(CAR_WIDTH+2)))
						cardata[i][6] = !(cardata[i][6]);		
			break;
			
		case GP142_PERIODIC:
			/* Timer interval event.  Update everything	*/
			/* that needs to change as time advances.	*/

			/* If a car is dead, stop it from moving */
			for (i=0; i<num_vehicles; i++) {
				if (cardata[i][7] == FALSE)
					cardata[i][5] = FALSE;
			}
			
			/* If the tank is ready to fire (tank_engaged == TRUE) move the tank toward the target */
			if (tank_engaged)
				move_tank(&tank_x, &tank_y, cardata[target][0]);

			/* If the tank is not engaged but is on the screen, move it to the right */
			if (!(tank_engaged) && (tank_x > - GP142_XMAX) && (tank_x < GP142_XMAX+TANK_WIDTH))
				move_tank(&tank_x, &tank_y, GP142_XMAX+TANK_WIDTH);

			/* If the tank is engaged and it is at one of the predetermined points on the screen, */
			/* let the tank fire */
			if (tank_engaged && (tank_x % (GP142_XMAX/4) == 0)) 
				tank_fired = TRUE;

			/* If the tank has fired, increment the animation sequence until the target is hit */
			if (tank_fired)
				if (fire_frame > TOTAL_FRAMES-1) {
					cardata[target][7] = FALSE;
					fire_frame = 0;
					tank_fired = FALSE;

				}
				else 
					fire_frame++;

			/* If the target is dead, find a new target.  If no new targets are available, */
			/* tank_engaged = FALSE */
			if (!cardata[target][7]) {
				target_canidate_exists = FALSE;
				for (i=0; i<num_vehicles; i++) {
					if (cardata[i][6] && cardata[i][7]) {
						target = i;
						target_canidate_exists = TRUE;
					}
				} 
				if (!target_canidate_exists)
					tank_engaged = FALSE;
			}
			
			/* Move everything else */
			move_sun(&sun_x, &sun_y);
			move_traffic(num_vehicles, cardata);

			break;
			
		default:
			/* unknown event */
			break;
			
		}  /*end switch */
		
		/* Redraw everything */
		draw_landscape(sun_x, sun_y);
		draw_buttons();
		draw_traffic(num_vehicles, cardata);
		
		/* If the tank is on the screen, redraw it with the turret pointing toward the target */
		if ((tank_x > - (GP142_XMAX + TANK_WIDTH/2) && (tank_x < GP142_XMAX + TANK_WIDTH/2)))
			draw_tank(tank_x,tank_y,cardata[target][0],cardata[target][1]);

		/* If the tank has fired, draw the next frame in the firing sequence */
		if (tank_fired)
			fire_tank(tank_x, tank_y, cardata[target][0], cardata[target][1], fire_frame);
		
	}  /* end event loop */
	
	
	/* Termination: shut down graphics window and exit */
	GP142_close();
	return 0;
}  /*end main */


/*****************************************************************************

     BACKGROUND LANDSCAPE

*****************************************************************************/


/* Draw the background, including the sun at coordinates sun_x and sun_y */
void draw_landscape(int sun_x, int sun_y)
{
	int text_offset = -GP142_XMAX + 15;	/* horizontal offset of text */
	int text_size   = 12;				/* size of text font */

	/* Draw the sky */
	GP142_rectangleXY(ICE_BLUE, -GP142_XMAX, -GP142_YMAX,
		GP142_XMAX, GP142_YMAX, FILL);
	
	/* Draw the sun */
	GP142_circleXY(YELLOW, sun_x, sun_y , 20);
	
	/* Draw the ground, highway, and text legend */
	GP142_rectangleXY(17, -GP142_XMAX, -GP142_YMAX,
		GP142_XMAX, HORIZON, FILL);
	GP142_rectangleXY(WHITE, 
					-GP142_XMAX, HIGHWAY_CENTER - HIGHWAY_WIDTH,
					 GP142_XMAX, HIGHWAY_CENTER + HIGHWAY_WIDTH,
					 FILL);
	GP142_rectangleXY(BLACK, 
					-GP142_XMAX, HIGHWAY_CENTER,
					 GP142_XMAX, HIGHWAY_CENTER,
					 FILL);
	
	GP142_printfXY (BLACK, text_offset, -GP142_YMAX+30, text_size,
		"Select or deselect a car by clicking on it.");

	GP142_printfXY (BLACK, text_offset, -GP142_YMAX+20, text_size,
		"Click on a button to control selected car<s>.");
	
	
	/*Draw some mountains */
	GP142_triangleXY(MED_GRAY,
		-GP142_XMAX, HORIZON,
		GP142_XMAX, HORIZON,
		0, (4 * GP142_YMAX)/5 ,
		FILL);
	
	GP142_triangleXY(OLIVE,
		-( 5 * GP142_XMAX/4), HORIZON,
		(GP142_XMAX)/3  , HORIZON,
		-(3* GP142_XMAX)/4, (2 * GP142_YMAX)/3 ,
		FILL);
	
	GP142_triangleXY(FOREST_GREEN,
		-GP142_XMAX / 3 , HORIZON,
		GP142_XMAX, HORIZON,
		GP142_XMAX, (2 * GP142_YMAX)/3 ,
		FILL);
	
} /*end draw_landscape */



/*****************************************************************************

     BUTTONS AND MOUSE CLICKS

*****************************************************************************/


/* Draws the buttons */
void draw_buttons( )
{
	int i;
	int text_offset = MENU_LEFT+5;	/* offset of button text in button */
	int font_size = 12;				/* size of button text */
	
	/* draw rectangles for each button */
	for(i = 0; i < NUM_BUTTONS; i++) {
		GP142_rectangleXY(BLACK,
			MENU_LEFT,
			MENU_TOP - i * BUTTON_HEIGHT,
			MENU_LEFT + BUTTON_WIDTH,
			MENU_TOP - (i + 1)*BUTTON_HEIGHT, 1);
	}
	
	/* display button text */
	i = MENU_TOP - (BUTTON_HEIGHT/2) - (font_size/2);
	GP142_printfXY(FOREST_GREEN, text_offset, i, font_size, "Go");	
	
	i -= BUTTON_HEIGHT;
	GP142_printfXY(RED, text_offset, i, font_size, "Stop");
	
	i -= BUTTON_HEIGHT;
	GP142_printfXY(BLACK, text_offset, i, font_size, "Faster");
	
	i -= BUTTON_HEIGHT;
	GP142_printfXY(BLACK, text_offset, i, font_size, "Slower");
	
	i -= BUTTON_HEIGHT;
	GP142_printfXY(BLACK, text_offset, i, font_size, "Reverse");
	
	i -= BUTTON_HEIGHT;
	GP142_printfXY(BLACK, text_offset, i, font_size, "New Color");
	
	i -= BUTTON_HEIGHT;
	GP142_printfXY(RED, text_offset, i+BUTTON_HEIGHT/6, font_size, "Mobilize NATO");
	GP142_printfXY(RED, text_offset, i-BUTTON_HEIGHT/3, font_size, "Ground Force");

}

/* Classify mouse click at (mouse_x,mouse_y) and return kind */
int  classify_mouse_click(int mouse_x, int mouse_y)
{
	if ((mouse_x > MENU_LEFT) &&
		(mouse_x < MENU_LEFT + BUTTON_WIDTH) &&
		(mouse_y < MENU_TOP) &&
		(mouse_y > (MENU_TOP - NUM_BUTTONS*BUTTON_HEIGHT)))
		return BUTTON_CLICK;
	else
		return OTHER_CLICK;
}



/* Process button click at location x, y.  Update car speed,	*/
/* direction, color, and whether traffic is moving as needed.	*/
void handle_button(int x, int y, int num_vehicles, int cardata[][8], int *tank_engaged)
{
	int i;				/* all purpose iteration constant */
	int buttonNum;		/* number of selected button */

	/* Complain if x coordinate couldn't be inside a button */
	if (x < MENU_LEFT || x > MENU_LEFT + BUTTON_WIDTH) {
		something_is_wrong();
		return;
	}
	
	/* Convert y coordinate to button number from top. */
	buttonNum = (MENU_TOP - y)/BUTTON_HEIGHT;	 
	
			switch(buttonNum) {
			case 0:
				/* go */
				for (i=0; i<num_vehicles; i++)
					if ((cardata[i][6] == TRUE) && (cardata[i][7] == TRUE))
						cardata[i][5] = TRUE;
				break;
			case 1:
				/* stop */
				for (i=0; i<num_vehicles; i++)
					if ((cardata[i][6] == TRUE) && (cardata[i][7] == TRUE))
						cardata[i][5] = FALSE;
				break;
		
			case 2:
				/* speed up */
				for (i=0; i<num_vehicles; i++)
					if ((cardata[i][6] == TRUE) && (cardata[i][7] == TRUE))
						cardata[i][4] = cardata[i][4] + 1;
				break;
			case 3:
				/* slow down */
				for (i=0; i<num_vehicles; i++)
					if ((cardata[i][6] == TRUE) && (cardata[i][7] == TRUE)) {
						cardata[i][4] = cardata[i][4] - 1;
						if (cardata[i][4] < 0)
							cardata[i][4] = 0;
						}
				break;
		
			case 4:
				/* reverse */
				for (i=0; i<num_vehicles; i++)
					if ((cardata[i][6] == TRUE) && (cardata[i][7] == TRUE))
						cardata[i][2] = - cardata[i][2];
				break;

			case 5:
				/* change car color */
				for (i=0; i<num_vehicles; i++)
					if ((cardata[i][6] == TRUE) && (cardata[i][7] == TRUE))
						cardata[i][3] = (cardata[i][3]+1) % MAX_COLORS;
				break;

			case 6:
				/* bring on the tank */
				*tank_engaged = TRUE;
				break;

			default:
				/*Bad news if this is ever reached! */
				something_is_wrong();
				break;
			}
}



/*****************************************************************************

     TRAFFIC AND OTHER ANIMATED OBJECTS

*****************************************************************************/


/* Update sun coordinates sun_x, sun_y by one tick */
void move_sun(int *sun_x, int *sun_y) {
	/* Move the sun to the left.  If it reaches the edge, move it */
	/* to the other side and down a bit. */
	(*sun_x) --;
	if (*sun_x < -GP142_XMAX -30) {
		*sun_x = - *sun_x;
		(*sun_y) --;
	}
}


/* Update all vehicle x, y coordinates, given current x, y */
/* location, direction (+1 or -1), and speed in pixels per tick, */
void move_traffic(int num_vehicles, int cardata[][8]) {
	/* Note: If there is more than one vehicle, this should */
	/* be updated to adjust all vehicle coordinates */
	
	int i;				/* all-purpose iteration constant */
	int trial_x;		/* new value for x coordinate */

	/* move vehicle.  If it moves off the edge, drives on from other */
	/* side of screen  */
	for (i=0; i<num_vehicles; i++) 
		if (cardata[i][5] == TRUE) {
			trial_x = cardata[i][0] + (cardata[i][2] * cardata[i][4]);
			if (trial_x > GP142_XMAX + CAR_WIDTH)
				cardata[i][0] = -GP142_XMAX - CAR_WIDTH/2;
			else if (trial_x < -GP142_XMAX - CAR_WIDTH)
				cardata[i][0] = GP142_XMAX + CAR_WIDTH/2;
			else
				cardata[i][0] = trial_x;
		} /* if statement */
}


/* move the tank two pixels to the right.  If the tank moves completely off the screen */
/* move it to the other side of the screen, and lower it by one pixel */
void move_tank(int *x, int *y, int target_x) {
	/* Move the tank to the right.  If it reaches the edge, move it */
	/* to the other side and down a bit. */
	if (*x < target_x) 
		(*x) = (*x) + 2;
	else (*x) --;
	if ((abs(*x)) > GP142_XMAX + TANK_WIDTH) {
		(*x) = - *x;
		(*y) --;
	}
}


/* Draw all vehicles given their current x, y coordinates, color, whether  */
/* it is selected, and whether or not it is alive  */
void draw_traffic(int num_vehicles, const cardata[][8]) {
	/* Note: needs to be changed if there is more than */
	/* one vehicle */
	int i;
	for (i=0; i<num_vehicles; i++) {
		if (cardata[i][2] == -1)
			draw_car(cardata[i][0],cardata[i][1]-HIGHWAY_WIDTH,cardata[i][3],cardata[i][6],cardata[i][7]);
		else if (cardata[i][2] == 1)
			draw_car(cardata[i][0],cardata[i][1],cardata[i][3],cardata[i][6],cardata[i][7]);
	}
}


/* Draw a single car of the given color with horizontal center x*/
/* and bottom y, draws red or black flag depending on whether it is selected,*/
/* and draws fire if car is dead */
void draw_car(int x, int y, int color, int selected, int alive) {
	if (!alive)
		y = y + 4;
	GP142_rectangleXY(color, x - CAR_WIDTH/2, y + CAR_HEIGHT/4,
						x + CAR_WIDTH/2, y + CAR_HEIGHT*3/4, FILL);
	GP142_rectangleXY(color, x - CAR_WIDTH/6, y + CAR_HEIGHT*3/4,
						x + CAR_WIDTH/6, y + CAR_HEIGHT, FILL);
	GP142_circleXY(BLACK, x - CAR_WIDTH/4, y + CAR_HEIGHT/4, CAR_HEIGHT/4);
	GP142_circleXY(BLACK, x + CAR_WIDTH/4, y + CAR_HEIGHT/4, CAR_HEIGHT/4);
	GP142_rectangleXY(color, x, y + CAR_HEIGHT*3/4,	x, y + CAR_HEIGHT + 1, FILL);
	GP142_rectangleXY(selected*2, x, y + CAR_HEIGHT + 1, x + 1, y + CAR_HEIGHT + 2, FILL);

	/* draw fire */
	if (!alive) {
		GP142_circleXY(BLACK, x, y + CAR_HEIGHT/2, CAR_HEIGHT/12);
		GP142_triangleXY(ORANGE, x - CAR_WIDTH/2 + (rand() % 5), y + CAR_HEIGHT/2 + (rand() % 3),
			x - CAR_WIDTH/4 + (rand() % 2 ), y + CAR_HEIGHT/2 + (rand() % 3),
			x - 2*CAR_WIDTH/5 + (rand() % 2), y + CAR_HEIGHT + 8 + (rand() % 5), FILL);
		GP142_triangleXY(YELLOW, x - 2*CAR_WIDTH/5 + (rand() % 3), y + CAR_HEIGHT/2 + (rand() % 3),
			x - CAR_WIDTH/4 + (rand() % 2 ), y + CAR_HEIGHT/2 + (rand() % 3),
			x - 3*CAR_WIDTH/10 + (rand() % 2), y + CAR_HEIGHT + 3 + (rand() % 5), FILL);
		GP142_triangleXY(ORANGE, x - 3*CAR_WIDTH/10 + (rand() % 5), y + 3*CAR_HEIGHT/4 + (rand() % 3),
			x + CAR_WIDTH/6 + (rand() % 5 ), y + 3*CAR_HEIGHT/4 + (rand() % 3),
			x + (rand() % 2), y + CAR_HEIGHT + 18 + (rand() % 5), FILL);
		GP142_triangleXY(YELLOW, x - CAR_WIDTH/6 + (rand() % 5), y + 3*CAR_HEIGHT/4 + (rand() % 3),
			x + CAR_WIDTH/6 + (rand() % 5 ), y + 3*CAR_HEIGHT/4 + (rand() % 3),
			x + (rand() % 2), y + CAR_HEIGHT + 8 + (rand() % 5), FILL);
		GP142_triangleXY(ORANGE, x + CAR_WIDTH/2 + (rand() % 5), y + CAR_HEIGHT/2 + (rand() % 3),
			x + CAR_WIDTH/4 + (rand() % 2 ), y + CAR_HEIGHT/2 + (rand() % 3),
			x + 2*CAR_WIDTH/5 + (rand() % 2), y + CAR_HEIGHT + 15 + (rand() % 5), FILL);
		GP142_triangleXY(ORANGE, x + CAR_WIDTH/6 + (rand() % 5), y + 3*CAR_HEIGHT/4 + (rand() % 3),
			x + 2*CAR_WIDTH/6 + (rand() % 5 ), y + 3*CAR_HEIGHT/4 + (rand() % 3),
			x + CAR_WIDTH/6 + (rand() % 2), y + CAR_HEIGHT + 8 + (rand() % 10), FILL);
		GP142_triangleXY(YELLOW, x + 2*CAR_WIDTH/5 + (rand() % 3), y + CAR_HEIGHT/2 + (rand() % 3),
			x + CAR_WIDTH/4 + (rand() % 2 ), y + CAR_HEIGHT/2 + (rand() % 3),
			x + 3*CAR_WIDTH/10 + (rand() % 2), y + CAR_HEIGHT + 3 + (rand() % 5), FILL);
	}
}


/* Draws the tank of coordinates x, y, where center is x and bottom is y */
/* draws turret in direction of target_x, target_y */
void draw_tank(int x, int y, int target_x, int target_y) {
	
	double sinx;		/* sine of x */
	sinx = sin(3.1415*((double) target_x - x)/((double) 3.0*GP142_XMAX));
		
		GP142_circleXY(BLACK, x, y + TANK_HEIGHT/8, TANK_HEIGHT/8);
		GP142_circleXY(BLACK, x - TANK_WIDTH/8, y + TANK_HEIGHT/8, TANK_HEIGHT/8);
		GP142_circleXY(BLACK, x + TANK_WIDTH/8, y + TANK_HEIGHT/8, TANK_HEIGHT/8);
		GP142_circleXY(BLACK, x - TANK_WIDTH*2/8, y + TANK_HEIGHT/8, TANK_HEIGHT/8);
		GP142_circleXY(BLACK, x + TANK_WIDTH*2/8, y + TANK_HEIGHT/8, TANK_HEIGHT/8);
		GP142_circleXY(BLACK, x - TANK_WIDTH*3/8, y + TANK_HEIGHT/8, TANK_HEIGHT/8);
		GP142_circleXY(BLACK, x + TANK_WIDTH*3/8, y + TANK_HEIGHT/8, TANK_HEIGHT/8);

		GP142_rectangleXY(FOREST_GREEN, x - TANK_WIDTH/2, y + TANK_HEIGHT/6,
						x + TANK_WIDTH/2, y + TANK_HEIGHT/2 + TANK_HEIGHT/16, FILL);

		GP142_triangleXY(FOREST_GREEN, x - TANK_WIDTH/2, y + TANK_HEIGHT/4,
			x - TANK_WIDTH/2, y + TANK_HEIGHT/2,
			x - TANK_WIDTH/2 - TANK_WIDTH/6, y + TANK_HEIGHT/2,
			FILL);
		GP142_triangleXY(FOREST_GREEN, x + TANK_WIDTH/2, y + TANK_HEIGHT/4,
			x + TANK_WIDTH/2, y + TANK_HEIGHT/2,
			x + TANK_WIDTH/2 + TANK_WIDTH/6, y + TANK_HEIGHT/2,
			FILL);
		GP142_triangleXY(FOREST_GREEN, x - TANK_WIDTH/2, y + TANK_HEIGHT/2,
			x - TANK_WIDTH/2, y + TANK_HEIGHT/2 + TANK_HEIGHT/16,
			x - TANK_WIDTH/2 - TANK_WIDTH/9, y + TANK_HEIGHT/2,
			FILL);
		GP142_triangleXY(FOREST_GREEN, x + TANK_WIDTH/2, y + TANK_HEIGHT/2,
			x + TANK_WIDTH/2, y + TANK_HEIGHT/2 + TANK_HEIGHT/16,
			x + TANK_WIDTH/2 + TANK_WIDTH/12, y + TANK_HEIGHT/2,
			FILL);
		GP142_rectangleXY(FOREST_GREEN, x - TANK_WIDTH/4, y + TANK_HEIGHT/2,
			x + TANK_WIDTH/4, y + 13*TANK_HEIGHT/16, FILL);

		GP142_lineXY(FOREST_GREEN, x + (int) (sinx*TANK_WIDTH/4), y + TANK_HEIGHT*6/8,
				x + (int) (sinx*TANK_WIDTH), y + 4*TANK_HEIGHT/3, FILL);		
}


/* draws a mortar shell between the tank turret and the target, at a position determined */
/* by the frame number */
void fire_tank(int x, int y, int target_x, int target_y, int frame)
{	
	double sinx;		/* sine of x */

	sinx = sin(3.1415*((double) target_x - x)/((double) 3.0*GP142_XMAX));

	GP142_rectangleXY(BLACK, x + (frame*(target_x-(x+(int) (sinx*TANK_WIDTH)))/6) 
								+ (int) (sinx*TANK_WIDTH),
					  y + 4*TANK_HEIGHT/3 + (frame*(target_y-(y+4*TANK_HEIGHT/3))/6),
					  x + (frame*(target_x-(x+(int) (sinx*TANK_WIDTH)))/6)
								+ (int) (sinx*TANK_WIDTH + 2),
					  y + 4*TANK_HEIGHT/3 + (frame*(target_y-(y+4*TANK_HEIGHT/3))/6) + 2, FILL);


}


/*****************************************************************************

     OTHER

*****************************************************************************/


/* Initialize random number generator */
/* [Ignore or delete if your program  */
/* doesn't user random numbers.] */
void initialize_random_numbers() {
	time_t t;			/* current time */
	srand (time(&t));
}



/* Display error message if internal error is detected. */
void something_is_wrong (void) {
	GP142_printfXY (RED, 0, 0, 16, "BAAAADDD NEWS!");
}